
#include "main.h"

static uint8_t bufEntry[500000];
static uint8_t bufRaw[500000];
static uint8_t bpp;
static uint32_t posEntry = 0, posRaw = 0;
static char fontStr[100];
static char fileStr[100];

static uint16_t minCar =  32;     //id car min INCLU
static uint16_t maxCar =  255;    //id car max INCLU
static uint16_t tailleFont = 15;
static SDL_Surface *ptrScreen = NULL;
#define FONT_NAME "font.ttf"



static void DisplayInit(void);
static uint32_t GetPx(SDL_Surface *surface, uint16_t x, uint16_t y);
static uint16_t GetHeighFont(TTF_Font *ptrFont);
static void SavePseudoBlendedCar1Bpp(TTF_Font *ptrFont, uint16_t idCar);
static void SavePseudoBlendedCar4Bpp(TTF_Font *ptrFont, uint16_t idCar);
static void CopyToFile(FILE *file);
static void CreateFontFile(void);




//cmd font.ttf h minCar maxCar

int main(int argc, char** argv) {

  if(argc != 7) {
    printf("usage: %s fontName.ttf bpp fontHeight minCar maxCar outFile.c\n", argv[0]);
    exit(-1);
  }

  strncpy(fontStr, argv[1], 100);

  bpp = atoi(argv[2]);
  switch(bpp) {
    case 1:
    case 4:
      break;
    default:
      exit(-1);
  }
  tailleFont = atoi(argv[3]);
  minCar = atoi(argv[4]);
  maxCar = atoi(argv[5]);

  strncpy(fileStr, argv[6], 100);

  DisplayInit();
  CreateFontFile();
  return 0;
}



/*
*   Private functions
*
*
*
*/
static uint32_t GetPx(SDL_Surface *surface, uint16_t x, uint16_t y) {
  if(surface == NULL) return 0;
  return *((uint32_t*)(surface->pixels) + x + y * surface->w);
}

static void DisplayInit(void) {

  if(SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    printf( "SDL Initialization failed: %s\n", SDL_GetError());
    exit(1);
  }

  atexit(SDL_Quit);
  SDL_putenv((char*)"SDL_VIDEO_WINDOW_POS=center");
  ptrScreen = SDL_SetVideoMode(320, 240, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
  if(ptrScreen == NULL) {
    printf("Cannot rendering SDL window\n");
    exit(1);
  }

  if(TTF_Init() < 0) {
    printf("Cannot start SDL_ttf: %s\n", TTF_GetError());
    exit(1);
  }
}

static uint16_t GetHeighFont(TTF_Font *ptrFont) {
  uint16_t hCar = 0;
  SDL_Color color = {255, 255, 255};
  SDL_Surface *surface = TTF_RenderText_Blended(ptrFont, "Mp", color);
  if(surface) {
    hCar = surface->h;
    SDL_FreeSurface(surface);
  }
  else {
    exit(1);
  }
  return hCar;
}

static void SavePseudoBlendedCar1Bpp(TTF_Font *ptrFont, uint16_t idCar) {

  unsigned char str[2] = {0, 0};
  uint16_t x, y;
  uint32_t colorPx;
  SDL_Color color = {255, 255, 255};
  str[0] = idCar;

  SDL_Surface *surface = TTF_RenderText_Blended(ptrFont, (char*)str, color);

  if(surface) {

    bufEntry[posEntry++] = surface->w;
    bufEntry[posEntry++] = (posRaw >> 16) & 0xFF;
    bufEntry[posEntry++] = (posRaw >> 8) & 0xFF;
    bufEntry[posEntry++] = posRaw & 0xFF;

    char mask;
    unsigned char byte = 0;
    char posBit = 7;

    for(y = 0; y < surface->h; y++) {
      for(x = 0; x < surface->w; x++) {

        colorPx = GetPx(surface, x, y);
        if(colorPx < 0x88888888) mask = 0;
        else mask = 1;

        mask <<= posBit;
        byte |= mask;
        posBit--;

        if(posBit < 0) {
          posBit = 7;
          bufRaw[posRaw++] = byte;
          byte = 0;
        }
      }
    }

    if(posBit != 7) {
        bufRaw[posRaw++] = byte;
    }
    SDL_FreeSurface(surface);
  }
}

static void SavePseudoBlendedCar4Bpp(TTF_Font *ptrFont, uint16_t idCar) {

  unsigned char str[2] = {0, 0};
  uint16_t x, y;
  uint32_t colorPx;
  SDL_Color color = {255, 255, 255};
  str[0] = idCar;

  SDL_Surface *surface = TTF_RenderText_Blended(ptrFont, (char*)str, color);

  if(surface) {

    bufEntry[posEntry++] = surface->w;
    bufEntry[posEntry++] = (posRaw >> 16) & 0xFF;
    bufEntry[posEntry++] = (posRaw >> 8) & 0xFF;
    bufEntry[posEntry++] = posRaw & 0xFF;

    char mask;
    unsigned char byte = 0;
    char posBit = 4;

    for(y = 0; y < surface->h; y++) {
      for(x = 0; x < surface->w; x++) {

        colorPx = GetPx(surface, x, y);
        if(colorPx < 0x08FFFFFF) mask = 0;
        else if(colorPx < 0x18FFFFFF) mask = 1;
        else if(colorPx < 0x28FFFFFF) mask = 2;
        else if(colorPx < 0x38FFFFFF) mask = 3;
        else if(colorPx < 0x48FFFFFF) mask = 4;
        else if(colorPx < 0x58FFFFFF) mask = 5;
        else if(colorPx < 0x68FFFFFF) mask = 6;
        else if(colorPx < 0x78FFFFFF) mask = 7;
        else if(colorPx < 0x88FFFFFF) mask = 8;
        else if(colorPx < 0x98FFFFFF) mask = 9;
        else if(colorPx < 0xA8FFFFFF) mask = 10;
        else if(colorPx < 0xB8FFFFFF) mask = 11;
        else if(colorPx < 0xC8FFFFFF) mask = 12;
        else if(colorPx < 0xD8FFFFFF) mask = 13;
        else if(colorPx < 0xE8FFFFFF) mask = 14;
        else mask = 15;

        mask <<= posBit;
        byte |= mask;
        posBit -= 4;

        if(posBit < 0) {
          posBit = 4;
          bufRaw[posRaw++] = byte;
          byte = 0;
        }
      }
    }

    if(posBit != 7) {
        bufRaw[posRaw++] = byte;
    }
    SDL_FreeSurface(surface);
  }
}



static void CopyToFile(FILE *file) {

  uint16_t cnt;

  fprintf(file, "  /*index table: glyph width, offset hl, lh, ll*/\n");
  for(cnt = 0; cnt < posEntry; cnt += 4) {
    fprintf(file, "  0x%02X, 0x%02X, 0x%02X, 0x%02X,\n", bufEntry[cnt], bufEntry[cnt+1], bufEntry[cnt+2], bufEntry[cnt+3]);
  }

  fprintf(file, "  /*glyph raw*/");
  for(cnt = 0; cnt < posRaw; cnt ++) {

    if(cnt % 20 == 0) fprintf(file, "\n  ");
    if(cnt == posRaw - 1) {
      fprintf(file, "0x%02X", bufRaw[cnt]);
    }
    else {
      fprintf(file, "0x%02X, ", bufRaw[cnt]);
    }

  }
}

static void CreateFontFile(void) {

  FILE *file = NULL;
  TTF_Font *ptrFont = NULL;
  uint16_t idCar;

  ptrFont = TTF_OpenFont(fontStr, tailleFont);

  if(ptrFont != NULL) {
    file = fopen(fileStr, "w+");

    if(file != NULL) {


      fprintf(file, "\n#include \"resources.h\"\n\n");
      fprintf(file, "const uint8_t font[] = {\n");
      fprintf(file, "  %d,  /*font type*/\n", bpp == 1? 1:2);
      fprintf(file, "  %d, /*first car ID*/\n", minCar);
      fprintf(file, "  %d, /*number of car*/\n", maxCar);
      fprintf(file, "  %d, /*font height*/\n", GetHeighFont(ptrFont));

      for(idCar = minCar; idCar <= maxCar; idCar++) {
        if(bpp == 1) SavePseudoBlendedCar1Bpp(ptrFont, idCar);
        else if(bpp == 4) SavePseudoBlendedCar4Bpp(ptrFont, idCar);
      }
      CopyToFile(file);

      fprintf(file, "\n};\n");
      fclose(file);
    }
  }
  else {

    exit(1);
  }
}
