
#include "main.h"


static SDL_Surface *ptrScreen = NULL;

static void DisplayInit(void);
static uint32_t GetPx(SDL_Surface *surface, uint16_t x, uint16_t y);
static void SaveImg(char *str);



int main(int argc, char** argv) {

  //if(argc != 5) {
    //printf("usage: %s fontName.ttf fontHeight minCar maxCar\n", argv[0]);
    //exit(-1);
  //}


  DisplayInit();

  SaveImg("img.png");

  return 0;
}






#define NB_MAX_COL 512
uint32_t colors[NB_MAX_COL];
uint32_t nbCol;


static int FindColor(uint32_t color) {
  int cnt;
  for(cnt = 0; cnt < nbCol; cnt++) {
    if(colors[cnt] == color) return cnt;
  }
  return -1;
}

static void AddColor(uint32_t color) {
  int id = FindColor(color);
  if(id < 0 && nbCol < NB_MAX_COL) colors[nbCol++] = color;
}

static void CreateLut(SDL_Surface *img) {

  int x, y;
  nbCol = 0;

  for(y = 0; y < img->h; y++) {
    for(x = 0; x < img->w; x++) {
      AddColor(GetPx(img, x, y));
    }
  }
}

static void SaveLut(FILE *file) {

  int cnt;

  fprintf(file, "\n  /*Color LUT*/\n");
  fprintf(file, "\n  %d, /*Number of colors*/\n", nbCol);

  for(cnt = 0; cnt < nbCol; cnt++) {
    fprintf(file, "  0x%02X, 0x%02X, 0x%02X,\n", (colors[cnt] >> 16) & 0xFF, (colors[cnt] >> 8) & 0xFF, colors[cnt] & 0xFF);
  }
}


static void SaveSprite(FILE *file, SDL_Surface *img) {

  int cnt;
  uint16_t x, y;

  fprintf(file, "\n  /*Sprite raw*/\n");
  cnt = 0;
  for(y = 0; y < img->h; y++) {
    for(x = 0; x < img->w; x++) {
      if(cnt % 20 == 0) fprintf(file, "\n  ");
      fprintf(file, "0x%02X, ", FindColor(GetPx(img, x, y)));
      cnt++;
    }
  }

}



static void SaveImg(char *str) {

  FILE *file = NULL;
  SDL_Surface *img;

  file = fopen("sprite.txt", "w+");

  if(file) {

    img = IMG_Load(str);

    fprintf(file, "\n#include \"resources.h\"\n\n");
    fprintf(file, "const uint8_t sprite[] = {\n");
    fprintf(file, "  %d, /*bpp*/\n", 8);
    fprintf(file, "  %d, /*width*/\n", img->w);
    fprintf(file, "  %d, /*height*/\n", img->h);

    CreateLut(img);
    SaveLut(file);

    SaveSprite(file, img);

    SDL_FreeSurface(img);

    fprintf(file, "\n};\n");
    fclose(file);
  }
}








static uint32_t GetPx(SDL_Surface *surface, uint16_t x, uint16_t y) {
  if(surface == NULL) return 0;
  return *((uint32_t*)(surface->pixels) + x + y * surface->w);
}

static void DisplayInit(void) {

  if(SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    printf( "SDL Initialization failed: %s\n", SDL_GetError());
    exit(1);
  }

  atexit(SDL_Quit);
  SDL_putenv((char*)"SDL_VIDEO_WINDOW_POS=center");
  ptrScreen = SDL_SetVideoMode(320, 240, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
  if(ptrScreen == NULL) {
    printf("Cannot rendering SDL window\n");
    exit(1);
  }
}
